#include "colors.inc"
#include "stones.inc"  
#include "metals.inc" 
#include "woods.inc"

/*
 * Stirling Engine, Copyright 2005 Stefan Runkel 
 * If you run this file, you get one image of the maschine.
 * If you run the ini File, you will get many images which form an animation.
 */

light_source {
   <0,1000,-1000>, rgb <1,1,1>
}            

#declare CameraSpline =
  spline {
    /* 
    20 Geraden (0..19) im Zeit-Intervall 0..9.
    Davon  3 Geraden mit Intervall 1; Rest: 9 -3 = 6
    Davon  4 mit Intervall m = 2n =  8 n
    Davon 13 mit Intervall n      = 13 n
    n= 6/21 ; m= 12/21 
    
    */
     0.00, <0,12.5,-25>                     //       3 Umdrehungen Stillstehen A
     1.00, <0,12.5,-25>                     //  0 3  3 Umdrehungen Stillstehen E   
     1.29, <-11.62,12.5,-22.14>             //  1 n
     1.86, <-10,12.5,-8>                    //  2 m  Bewegung auf Nahaufnahme Zylinder; A
     2.86, <-10,12.5,-8>                    //  3 3  Nahaufnahme Zylinder 3 Umdrehungen Stillstehen E
     3.43, <-11.62,12.5,-22.14>             //  4 m
     3.71, <-20.57,12.5,-14.2>              //  5 n
     4.00, <-24.82,12.5,-3.01>              //  6 n
     4.29, <-23.38,12.5,8.87>               //  7 n  Umlauf bis Nahaufnahme Kurbelwelle P1
     4.57, <-16.58,12.5,18.71>              //  8 n  Umlauf bis Nahaufnahme Kurbelwelle P2
     4.86, <-5.98,12.5,24.27>               //  9 n  Umlauf bis Nahaufnahme Kurbelwelle P3
     5.14, <5.98,12.5,24.27>                // 10 n  Umlauf bis Nahaufnahme Kurbelwelle P4
     5.43, <16.58,12.5,18.71>               // 11 n  Umlauf bis Nahaufnahme Kurbelwelle P5  
     5.71, <23.38,12.5,8.87>                // 12 n
     6.00, <24.82,12.5,-3.01>               // 13 n
     6.29, <20.57,12.5,-14.2>               // 14 n
     6.86, <8,12.5,-9>                      // 15 m  Umlauf bis Nahaufnahme Kurbelwelle P6; A
     7.86, <8,12.5,-9>                      // 16 3  Nahaufnahme Kurbelwelle 3 Umdrehungen Stillstehen E  
     8.43, <20.57,12.5,-14.2>               // 17 m
     8.71, <11.62,12.5,-22.14>              // 18 n  Umlauf bis Anfang p1
     9.00, <0,12.5,-25>                     // 19 n  Umlauf bis Anfang p2   
  }

      
camera {
  //location <-10,10,-8>   
  location CameraSpline(clock*9)
  //location <0,65,0>
  look_at <0,0,0>
  //rotate y* clock * 360
 }
   
union{
  //plane {z,40}  
  plane {y,-8}          
  texture {   
    pigment{ color <.8,.8,.5>}
  } 
}

  
#declare Holes=5;  

#declare Flywheel=object { 
  union{
    difference{          
      cylinder{ <0,0,-.5>,<0,0,.5>,3.5}                                                      // Material
      difference{ cylinder{ <0,0,.2>,<0,0,.6>,3.3}   cylinder{ <0,0,.2>,<0,0,.6>,.6}}        // Absatz hinten
      difference{ cylinder{ <0,0,-.2>,<0,0,-.6>,3.3} cylinder{ <0,0,-.2>,<0,0,-.6>,.6}}      // Absatz vorne 
      difference{ cylinder{<0,0,-1>,<0,0,1>,3}       cylinder{<0,0,-1>,<0,0,1>,0.8}}         // Ausschnitt
    }   
    cylinder{<0,0,-.6>,<0,0,.6>,.1}                                                          // Achsenbohrung
    #declare Count=0;
    #while (Count < 360)  
    union{                                                                                  
      box {<-.26,.7,-.2>,<.26,3.1,.2> }                                           // Speiche     
      difference{ box{<-0.76,2.5,-.2><-.26,3,.2>} cylinder{<-0.76,2.5,-.3>,<-0.76,2.5,.3>,.5}}
      difference{ box{<0.76,2.5,-.2><.26,3,.2>}   cylinder{<0.76,2.5,-.3>,<0.76,2.5,.3>,.5}}
      rotate (Count - 180/Holes) *z
    }
    #declare Count=Count+ 360 / Holes;
    #end     
    texture{T_Brass_1B}
  }                            
}
     
     
#declare CrankP=object{
  union {
    cylinder {<0,0,-1>,<0,0,-1.2>,.15 }
    cylinder {<0,0,-1>,<0,0,-1.2>,.15 translate y*1}
    box {<-.15,0,-1>,<.15,1,-1.2>}
    texture{T_Brass_3C}  
  }

} 

#declare Crank=object{  
  difference {
    union {                                
      cylinder {<0,0,-2.55>,<0,0,2.4>,.1}                                               // Achse
      object {CrankP}  
      object {CrankP rotate 90*z } 
      object {CrankP rotate 90*z translate -.8*z}
      object {CrankP translate z*.8} 
      cylinder {<0,0,-1.22>,<0,0,-.19>,.1 translate y*1}                                // Achse Excenter 1
      cylinder {<0,0,-1.22>,<0,0,-.19>,.1 translate y*1 rotate 90*z translate -.8*z}    // Achse Excenter 2 
    }
    cylinder {<0,0,-.99>,<0,0,-.401>,.102 }                                             // Abschnitt Achse Excenter 1
    cylinder {<0,0,-.99>,<0,0,-.401>,.102 translate -.8*z}                              // Abschnitt Achse Excenter 2          
  }
  texture{T_Chrome_1B} 
  translate -1*z 
}     

     
#declare Base=object{
  difference{
    box {<-6,0,-4>,<3,.5,3> } 
    cylinder {<-6,.5,-4>,<6,.5,-4>,.2}
    cylinder {<-6,.5,3>,<6,.5,3>,.2} 
    cylinder {<-6,.5,-4>,<-6,.5,3>,.2} 
    cylinder {<3,.5,-4>,<3,.5,3>,.2}
    translate y * -4.5
    texture{T_Wood34 scale 4 rotate y*15}
  }

}
  
  
#declare StandP=object{ 
  difference{
  union{
    box {<-.2,0,0>,<.2,-4,.3>}
    box {<-.4,-2,0>,<.4,-4,.3>} 
    cylinder {<0,0,0>, <0,0,.3>,.3 }
    cylinder {<0,-1.5,0>, <0,-1.5,.3>,1 }
    cylinder {<0,-4,0>, <0,-4,.3>,1.2 }  
  }
  cylinder {<0,-1.5,-.1>, <0,-1.5,.4>,.8 }
  cylinder {<0,-4,-.1>, <0,-4,.4>,.8 }
  cylinder {<-.65,-2.65,-.1>, <-.65,-2.65,.4>,.3 }
  cylinder {<.65,-2.65,-.1>, <.65,-2.65,.4>,.3 }          
  } 
  texture{T_Brass_3C}
}

              
#declare Stand=object{ 
  union{  
    object { StandP translate z*1}
    object { StandP translate z*-3.5}
  }          
}

#declare Screw=object{ 
  difference{
    union { cylinder{<-1.4,.2,-.6><-1.4,.3,-.6>,.1} cylinder{<-1.4,-.07,-.6><-1.4,.3,-.6>,.05} } // Kopf, Schaft
    box {<-.11,.25,-0.015>,<.11,.35,.015> rotate 45*y translate -1.4*x translate -.6 *z}        // Schlitz
  }
  texture{T_Chrome_1B}
}
            
#declare RodL=object{
  union {
    //cylinder {<0,0,-1>,   <0,0,-1.2>,  .15 } // Lager an Kurbelwelle  
    difference { box {<-.15,-.3,-1>,<.15,.3,-1.2>} box {<-.01,-.4,-.9>,<.01,.4,-1.3>}}
    object {Screw translate 1.4 *x translate -.5 *z rotate z*-90 translate y*.2  translate -.06 *x}  
    object {Screw translate 1.4 *x translate -.5 *z rotate z*-90 translate y*-.2 translate -.06 *x}
  }
}            
            
#declare Rod=object{
#declare LRod=3;
  union{
    object { RodL }                          // Lager an Kurbelwelle
    cylinder {<-3,0,-1>,  <-3,0,-1.2>, .15 } // Lager an Kolben
    cylinder {<0,0,-1.1>, <-3,0,-1.1>, .08 } // Verbindungsstange
    translate z*-.6
    texture{T_Brass_3C}
  }
}    

#declare Rod1=object{
#declare LRod=3;
  union{
    object { RodL }                          // Lager an Kurbelwelle
    cylinder {<-3,0,-1.025>,  <-3,0,-1.175>, .075 }
    cylinder {<0,0,-1.1>, <-3,0,-1.1>, .08  }
    translate z*-.6
    texture{T_Brass_3C}
  }
}              
     
#declare Piston=object{  
  union{
  difference{
    cylinder {<0,0,0>,<-1.5,0,0>,.3 } 
    torus{.3,.03 rotate z*90 translate x*-1.3}  
    torus{.3,.03 rotate z*90 translate x*-1.4}
    box  {<-.3,-.6,-.1>,<.1,.6,.1>}
    
    texture{T_Silver_1B}
  }
  cylinder {<-.2,0,-.3>,<-.2,0,.3>,.1 texture{T_Chrome_1B}} 
  }     
  translate z*-1.7 translate x* .2 
}  


#declare Piston1=object{  
  union{
  difference{
    cylinder {<0,0,0>,<-1.5,0,0>,.15 } 
    torus{.15,.015 rotate z*90 translate x*-1.3}  
    torus{.15,.015 rotate z*90 translate x*-1.4}
    box  {<-.3,-.6,-.075>,<.1,.6,.075>}
    texture{T_Silver_1B}
  }      
  cylinder {<-.2,0,-.151>,<-.2,0,.151>,.05 texture{T_Chrome_1B}} 
  }     
  translate z*-1.7 translate x* .2 
}              




#declare Cylinder=object{
  union{ 
  difference{ 
    union{         
      cylinder {<-1.6,0,0>,<.5,0,0>,.5 }   
      cylinder {<-1.5,0,0>,<-1.3,0,0>,.7 }
      cylinder {<-1.5,0,0>,<-1.4,0,0>,.7 translate x* .3}
      cylinder {<-1.5,0,0>,<-1.4,0,0>,.7 translate x* .5}
      cylinder {<-1.5,0,0>,<-1.4,0,0>,.7 translate x* .7}
      cylinder {<-1.5,0,0>,<-1.4,0,0>,.7 translate x* .9}
      cylinder {<-1.5,0,0>,<-1.4,0,0>,.7 translate x* 1.1}
      cylinder {<-1.5,0,0>,<-1.4,0,0>,.7 translate x* 1.3} 
      cylinder {<-1.5,0,0>,<-1.3,0,0>,.7 translate x* 1.5}
    }  
    cylinder {<-1.55,0,0>,<.6,0,0>,.3 }  
    box {<-1.51,.2,-.71>,<-1.29,1,-.5>}
    box {<-1.51,.2,-.71>,<-1.29,1,-.5> translate x* 1.5}
    box {<-1.51,.2,-.71>,<-1.29,1,-.5> translate y* -1.2}
    box {<-1.51,.2,-.71>,<-1.29,1,-.5> translate y* -1.2 translate x* 1.5}
    
    box {<-1.51,.2,-.71>,<-1.29,1,-.5> translate z*1.2}
    box {<-1.51,.2,-.71>,<-1.29,1,-.5> translate x* 1.5 translate z*1.2}
    box {<-1.51,.2,-.71>,<-1.29,1,-.5> translate y* -1.2 translate z*1.2}
    box {<-1.51,.2,-.71>,<-1.29,1,-.5> translate y* -1.2 translate x* 1.5 translate z*1.2}
  }
  cylinder{<-1.4,0,-.6><-1.4,-4,-.6>,.1}
  cylinder{<-1.4,0,-.6><-1.4,-4,-.6>,.1 translate z*1.2}
  cylinder{<-1.4,0,-.6><-1.4,-4,-.6>,.1 translate x* 1.5}
  cylinder{<-1.4,0,-.6><-1.4,-4,-.6>,.1 translate x* 1.5 translate z*1.2}  
  
  union{
    object{Screw}
    object{Screw translate z* 1.2}
    object{Screw translate x* 1.5}
    object{Screw translate x* 1.5 translate z*1.2}
  }     
  } 
  texture{T_Brass_3C}     
  translate z*-1.7 translate x* -3.75 
}              


#declare Cylinder1=object{
  difference{ 
    union{         
      cylinder {<-1.6,0,0>,<.5,0,0>,.2 }   
      cylinder {<-1.5,0,0>,<-1.3,0,0>,.3 }
      cylinder {<-1.5,0,0>,<-1.4,0,0>,.3 translate x* .3}
      cylinder {<-1.5,0,0>,<-1.4,0,0>,.3 translate x* .5}
      cylinder {<-1.5,0,0>,<-1.4,0,0>,.3 translate x* .7}
      cylinder {<-1.5,0,0>,<-1.4,0,0>,.3 translate x* .9}
      cylinder {<-1.5,0,0>,<-1.4,0,0>,.3 translate x* 1.1}
      cylinder {<-1.5,0,0>,<-1.4,0,0>,.3 translate x* 1.3} 
      cylinder {<-1.5,0,0>,<-1.3,0,0>,.3 translate x* 1.5}
    }  
    cylinder {<-1.55,0,0>,<.6,0,0>,.15 }  
  } 
  texture{T_Brass_3C}     
  translate z*-1.7 translate x* -3.75 
}   
     
#declare FlameMedia=
 sphere {0,1.1
   pigment {rgbf 1}
        interior {
                media {
                method 2 intervals 2 samples 3,3  // megapov, or increase intervals and samples
                absorption <.33,.5,.67>*2 emission <.33,.5,.67>*1.67
                scattering {2,<.75,.67,.5>*2 extinction .133}
                density {spherical turbulence <.1,.2,.15>*1.5
                        density_map {
                        [0 rgb 0]
												[.15 rgb <1.5,1,.5>*3]
												[.5 rgb <1.25,.5,.75>*2]
												[.7 rgb<0,.05,.5>*6]
                        } scale <.85,.9,.85> translate -.25*y}
                }
                media {
                method 2 intervals 2 samples 3,3  // megapov, or increase intervals and samples
                absorption <.36,.33,.167>*3 emission <.25,.5,.75>*1.133
                scattering {1,<.25,.33,.75>*3 extinction .67}
                density {spherical turbulence <.1,.2,.15>*.5
                        density_map {
                        [0 rgb 0][.3 rgb <0,.05,1.15>*6]
                        } scale <.7,.3,.7> translate -.9*y}
                }
       }
  scale <.9,3,.9>*.25 rotate 45*y translate .225*y
 hollow
 }

#declare SmallFlame= 
 light_source { <0,.5,0>, <.75,.5,.25> 
       area_light <.5,0,0>,<0,.5,0>,3,3 jitter adaptive 1
    orient circular
  looks_like {FlameMedia}
   media_interaction on media_attenuation on
   fade_distance 6 fade_power 2 
 }
       
#declare Burner= 
object {  
 union{
   cylinder {<-.65,-4,0>,<-.65,-3.4,0>,.5}
   cylinder {<-.65,-4,0>,<-.65,-2.9,0>,.05}
   cone     {<-.65,-3.4,0>,.5,<-.65,-3,0>,.05 }
   texture{T_Brass_3C}
 }
} 
       
 

             
union{  
  #declare R1 = seed(0);
  #declare C=0;
  #declare R2=0;
  #while (C < clock * 360)
    #declare R2 = rand(R1)*.3 + .7;
    #declare C=C+1; 
    
  #end
  union{    
    #declare Grad=mod (clock*360*9-75, 360);
    object {Flywheel}   
    object {Crank}
    rotate Grad*z
  }           
  
  #declare Xa   = sin(Grad*pi/180);
  #declare Ya   = cos(Grad*pi/180);
  #declare Xrod = (-1)*(sqrt((LRod*LRod)-(Ya*Ya))-Xa);
  #declare Xd   = Xa-Xrod;
  #declare Yd   = Ya;
  #declare Arod = asin(Ya/LRod)*180/pi;

  #declare Xa1   = sin((Grad+90)*pi/180);
  #declare Ya1   = cos((Grad+90)*pi/180);
  #declare Xrod1 = (-1)*(sqrt((LRod*LRod)-(Ya1*Ya1))-Xa1);
  #declare Xd1   = Xa1-Xrod1;
  #declare Yd1   = Ya1;
  #declare Arod1 = asin(Ya1/LRod)*180/pi;
  
  object {Rod     rotate z* Arod  translate Ya*y  translate -Xa*x}
  object {Rod1    rotate z* Arod1 translate Ya1*y translate -Xa1*x translate -.8*z}
  object {Piston  translate x* (Xrod-2*Xa)}
  object {Piston1 translate -.8*z translate x* (Xrod1-2*Xa1)} 
  object {Stand}
  object {Base} 
  object {Cylinder}
  object {Cylinder1 translate -.8*z}    
  object {Burner translate -3.8*x translate -1.22*z}  
  object {SmallFlame scale (.7 + R2*.3) translate -4.5*x translate -2.8*y translate -1.22*z}
  scale 2.3 
  translate y*3 translate x*2 
  rotate  35 *y  
  //rotate  -0 *y  
  
}                             

/*
union{
  #declare ctr = 0;
  #while (ctr < 1)
  sphere {
    CameraSpline(ctr*9),.5
    pigment { rgb <1,1,0> }
  }
  #declare ctr = ctr + 0.01;
  #end
}
*/   
